<?php

/**
 * Implementation of hook_drush_command()
 */
function ldapprov_drush_command() {
  $items = array();
  
  $items['ldapprov-export-all'] = array(
    'description' => 'Export all users in Drupal that are not in LDAP',
    'callback' => 'ldapprov_drush_export_all',
  );
  $items['ldapprov-check-all'] = array(
    'description' => 'Check all users in Drupal are in LDAP',
    'callback' => 'ldapprov_drush_check_all',
  );
  return $items;
}

function ldapprov_drush_export_all() {
  ldapprov_drush_scan_all('_ldapprov_drush_provision');
}

function _ldapprov_drush_provision($account, $ldap_record = NULL) {
  $drupal_user = user_load($account['uid']);
  if (!empty($ldap_record)) {
    $drupal_user->ldap_dn = $ldap_record['dn'];
    $drupal_user->ldap_authentified = TRUE;
    $edit = array(
      'ldap_dn' => $ldap_record['dn'],
      'ldap_authentified' => 1,
    );
    $result = (user_save($drupal_user, $edit) === FALSE) ? 'failed' : 'success';
    drush_log(dt('Linking @user to @dn', array('@user' => $account->name, '@dn' => $ldap_record['dn'])), '$result');
    return;
  }
  $edit = array();
  $status = ldapprov_provision_user($edit, $drupal_user) ? 'success' : 'failed';
  drush_log(dt('Provisioning @user', array('@user' => $drupal_user->name)), $status);
}

function ldapprov_drush_check_all() {
  return ldapprov_drush_scan_all();
}

function ldapprov_drush_scan_all($not_found_callback=NULL, $found_callback=NULL) {
  $sid = variable_get('ldapprov_server', NULL);
  if (empty($sid)) {
    drush_log(dt('No provisioning server configured.'), 'error');
    return;
  }

  $user_count = $no_ldap_dn = $found_count = $missing_count = 0;

  $server = new LDAPInterface(ldapauth_server_load($sid));
  if (!$server->connectAndBind($server->binddn, $server->bindpw)) {
    drush_log(dt('Could not bind to @server using DN @dn', array('@server' => $server->name, '@dn' => $server->binddn)), 'error');
    return;
  }

  // Needed for searching and construcing basenames
  $basedn = $server->getOption('basedn');
  $name_attr = $server->getOption('user_attr') ? $server->getOption('user_attr') : LDAP_DEFAULT_USER_ATTRIBUTE;


  $result = db_query("SELECT * FROM {users}");
  while ($user = db_fetch_array($result)) {
    // Skip Anonymous and UID 1
    if ($user['uid'] < 2) {
      continue;
    }

    // Come, mister tally man, tally me some users
    $user_count++;

    // Shoudl the user be in LDAP?
    $data = unserialize($user['data']);
    if (empty($data['ldap_dn'])) {
      $no_ldap_dn++;
      drush_log(dt("No LDAP DN for @user", array('@user' => $user['name'])), 'notice');
      $records = $server->search($basedn, "($name_attr={$user['name']})");
      if ($records['count'] > 0) {
        drush_log(dt("    but LDAP record found: @dn", array('@dn' => $records[0]['dn'])), 'notice');
      }
      if (!empty($not_found_callback)) {
        $not_found_callback($user, $records[0]);
      }
      continue;
    }

    // Search in LDAP,
    $record = $server->retrieveAttributes($data['ldap_dn']);
    if (empty($record)) {
      drush_log(dt("Missing @user (@dn)", array('@user' => $user['name'], '@dn' => $data['ldap_dn'])), 'notice');
      $missing_count++;
      if (!empty($not_found_callback)) {
        $not_found_callback($user, NULL);
      }
    }
    else {
      drush_log(dt("Found @user (@dn)", array('@user' => $user['name'], '@dn' => $data['ldap_dn'])), 'debug');
      $found_count++;
      if (!empty($found_callback)) {
        $found_callback($user);
      }
    }
  }
  $server->disconnect();

  // Report
  if ($no_ldap_dn > 0) {
    drush_log(dt('Found @found with no LDAP record', array('@found' => format_plural($no_ldap_dn, '1 user', '@count users'))), 'warning');
  }

  if ($missing_count > 0) {
    drush_log(dt('Found @found with LDAP records.', array('@found' => format_plural($found_count, '1 user', '@count users'))), 'warning');
    drush_log(dt('Missing @missing.', array('@missing' => format_plural($missing_count, '1 user', '@count users'))), 'warning');
    drush_log(dt("Run with --verbose to see the list of missing users."), 'warning');
  } else {
    drush_log(dt('Found all @found', array('@found' => format_plural($found_count, '1 user', '@count users'))), 'ok');
  }
}